{******************************************************************************}
{                                                                              }
{ HID API exerciser program                                                    }
{                                                                              }
{ The contents of this file are subject to the Mozilla Public License Version  }
{ 1.1 (the "License"); you may not use this file except in compliance with the }
{ License. You may obtain a copy of the License at http://www.mozilla.org/MPL/ }
{                                                                              }
{ Software distributed under the License is distributed on an "AS IS" basis,   }
{ WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for }
{ the specific language governing rights and limitations under the License.    }
{                                                                              }
{ The Original Code is HidTester.pas.                                          }
{                                                                              }
{ The Initial Developer of the Original Code is Robert Marquardt.              }
{ (robert_marquardt@gmx.de)                                                    }
{                                                                              }
{ Portions created by Robert Marquardt are                                     }
{ Copyright (c) 2003 Robert Marquardt.                                         }
{                                                                              }
{ Last modified: December 30, 2002                                             }
{                                                                              }
{******************************************************************************}
{                                                                              }
{ This program is a present to Jan Axelson, Lakeview Research.                 }
{ It is only available from Jan Axelson.                                       }
{                                                                              }
{******************************************************************************}

unit HidTester;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls,
  JvHidControllerClass, Hid;

type
  TBasicTestForm = class(TForm)
    PreparsedDataBtn: TButton;
    PreparsedDataLbl: TLabel;
    CapsBtn: TButton;
    CapsLbl: TLabel;
    ConfigurationBtn: TButton;
    ConfigurationLbl: TLabel;
    LinkCollectionNodesLbl: TLabel;
    LinkCollectionNodesBtn: TButton;
    ManufacturerStringLbl: TLabel;
    ManufacturerStringBtn: TButton;
    ProductStringLbl: TLabel;
    ProductStringBtn: TButton;
    SerialNumberStringLbl: TLabel;
    SerialNumberStringBtn: TButton;
    PhysicalDescriptorLbl: TLabel;
    PhysicalDescriptorBtn: TButton;
    HIDLanguagesLbl: TLabel;
    HIDLanguagesBtn: TButton;
    HIDStringsLbl: TLabel;
    HIDStringsBtn: TButton;
    procedure PreparsedDataBtnClick(Sender: TObject);
    procedure CapsBtnClick(Sender: TObject);
    procedure ConfigurationBtnClick(Sender: TObject);
    procedure LinkCollectionNodesBtnClick(Sender: TObject);
    procedure ManufacturerStringBtnClick(Sender: TObject);
    procedure ProductStringBtnClick(Sender: TObject);
    procedure SerialNumberStringBtnClick(Sender: TObject);
    procedure PhysicalDescriptorBtnClick(Sender: TObject);
    procedure HIDLanguagesBtnClick(Sender: TObject);
    procedure HIDStringsBtnClick(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  public
    TestDev: TJvHidDevice;
    PrepData: PHIDPPreparsedData;
    Config: THIDDConfiguration;
    Caps: THIDPCaps;
    LinkCollection: array of THIDPLinkCollectionNode;
    PhysicalDescriptor: TJvPhysicalDescriptor;
  end;

var
  TestForm: TBasicTestForm;

implementation

{$R *.dfm}

procedure TBasicTestForm.PreparsedDataBtnClick(Sender: TObject);
begin
  if PrepData <> nil then
    HidD_FreePreparsedData(PrepData);
  PrepData := nil;
  PreparsedDataBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    if HidD_GetPreparsedData(TestDev.HidFileHandle, PrepData) then
    begin
      PreparsedDataLbl.Caption := 'Preparsed data successfully accessed';
      PreparsedDataBtn.Tag := 2;
    end
    else
      PreparsedDataLbl.Caption := 'Preparsed data not accessible';
  end
  else
    PreparsedDataLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.CapsBtnClick(Sender: TObject);
var
  Ret: NTSTATUS;
begin
  if PreparsedDataBtn.Tag = 0 then
    PreparsedDataBtnClick(Self);
  CapsBtn.Tag := 1;
  FillChar(Caps, SizeOf(THIDPCaps), #0);
  if PreparsedDataBtn.Tag <> 2 then
    CapsLbl.Caption := 'Preparsed data not available to call HidP_GetCaps'
  else
  begin
    Ret := HidP_GetCaps(PrepData, Caps);
    if Ret = HIDP_STATUS_SUCCESS then
    begin
      CapsLbl.Caption := Format('successfully called (Usage=%u UsagePage=%u)',
        [Caps.Usage, Caps.UsagePage]);
      CapsBtn.Tag := 2;
    end
    else
      CapsLbl.Caption := Format('failed (%s)', [HidErrorString(Ret)]);
  end;
end;

procedure TBasicTestForm.ConfigurationBtnClick(Sender: TObject);
begin
  Config.cookie         := nil;
  Config.size           := 0;
  Config.RingBufferSize := 0;

  ConfigurationBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    if HidD_GetConfiguration(TestDev.HidFileHandle, Config, SizeOf(THIDDConfiguration)) then
    begin
      ConfigurationLbl.Caption := Format('successfully called (RingBufferSize=%u)',
        [Config.RingBufferSize]);
      ConfigurationBtn.Tag := 2;
    end
    else
    if GetLastError = ERROR_INVALID_FUNCTION then
      ConfigurationLbl.Caption := 'failed because of no configurations'
    else
      ConfigurationLbl.Caption := Format('failed unexpectedly (%s)', [SysErrorMessage(GetLastError)]);
  end
  else
    ConfigurationLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.LinkCollectionNodesBtnClick(Sender: TObject);
var
  Siz: ULONG;
  Ret: NTSTATUS;
begin
  if PreparsedDataBtn.Tag = 0 then
    PreparsedDataBtnClick(Self);
  if CapsBtn.Tag = 0 then
    CapsBtnClick(Self);
  LinkCollectionNodesBtn.Tag := 1;
  if PreparsedDataBtn.Tag <> 2 then
    LinkCollectionNodesLbl.Caption := 'Preparsed data not available to call HidP_GetLinkCollectionNodes'
  else
  if CapsBtn.Tag <> 2 then
    LinkCollectionNodesLbl.Caption := 'Caps not available to call HidP_GetLinkCollectionNodes'
  else
  begin
    Siz := Caps.NumberLinkCollectionNodes;
    SetLength(LinkCollection, Siz);
    Ret := HidP_GetLinkCollectionNodes(@LinkCollection[0], Siz, PrepData);
    if Ret = HIDP_STATUS_SUCCESS then
    begin
      LinkCollectionNodesLbl.Caption := Format('successfully called (%u nodes)', [Siz]);
      LinkCollectionNodesBtn.Tag := 2;
    end
    else
      LinkCollectionNodesLbl.Caption := Format('failed (%s)', [HidErrorString(Ret)]);
  end;
end;

procedure TBasicTestForm.ManufacturerStringBtnClick(Sender: TObject);
var
  Buffer: array [0..253] of WideChar;
begin
  FillChar(Buffer, SizeOf(Buffer), #0);
  ManufacturerStringBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    if HidD_GetManufacturerString(TestDev.HidFileHandle, Buffer, SizeOf(Buffer)) then
    begin
      ManufacturerStringLbl.Caption := Format('successfully called ("%s")',
        [Buffer]);
      ManufacturerStringBtn.Tag := 2;
    end
    else
      ManufacturerStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(GetLastError)]);
  end
  else
    ManufacturerStringLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.ProductStringBtnClick(Sender: TObject);
var
  Buffer: array [0..253] of WideChar;
begin
  FillChar(Buffer, SizeOf(Buffer), #0);
  ProductStringBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    if HidD_GetProductString(TestDev.HidFileHandle, Buffer, SizeOf(Buffer)) then
    begin
      ProductStringLbl.Caption := Format('successfully called ("%s")',
        [Buffer]);
      ProductStringBtn.Tag := 2;
    end
    else
      ProductStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(GetLastError)]);
  end
  else
    ProductStringLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.SerialNumberStringBtnClick(Sender: TObject);
var
  I: Integer;
  Len: Integer;
  IDs:    array [0..253] of WORD;
  Buffer: array [0..253] of WideChar;
  SerialNumber: string;
begin
  FillChar(Buffer, SizeOf(Buffer), #0);
  SerialNumberStringBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    if HidD_GetSerialNumberString(TestDev.HidFileHandle, Buffer, SizeOf(Buffer)) then
    begin
      // calculate length of StringDescriptor 0
      FillChar(IDs, SizeOf(IDs), $FF);
      Len := 0;
      HidD_GetIndexedString(TestDev.HidFileHandle, 0, PWideChar(@IDs), SizeOf(IDs));
      for I := High(IDs) downto 0 do
        if IDs[I] <> $FFFF then
        begin
          if IDs[I] = 0 then
            Len := I
          else
            Len := I+1;
          Break;
        end;
      SerialNumber := '';
      // compensate for buggy function
      for I := 0 to Len - 1 do
        if IDs[I] <> WORD(Buffer[I]) then
        begin
          SerialNumber := Buffer;
          Break;
        end;
      SerialNumberStringLbl.Caption := Format('successfully called ("%s")',
        [SerialNumber]);
      SerialNumberStringBtn.Tag := 2;
    end
    else
      SerialNumberStringLbl.Caption := Format('failed (%s)', [SysErrorMessage(GetLastError)]);
  end
  else
    SerialNumberStringLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.PhysicalDescriptorBtnClick(Sender: TObject);
var
  I: Integer;
begin
  PhysicalDescriptorBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    I := 1;
    SetLength(PhysicalDescriptor, 2048);
    while not HidD_GetPhysicalDescriptor(TestDev.HidFileHandle, PhysicalDescriptor[0], I*SizeOf(WORD)) do
    begin
      Inc(I);
      if (I > 2048) or (GetLastError <> ERROR_INSUFFICIENT_BUFFER) then
      begin
        I := 0;
        PhysicalDescriptorLbl.Caption := Format('failed (%s)', [SysErrorMessage(GetLastError)]);
        Break;
      end;
    end;
    if I > 0 then
    begin
      if (I = 1) and (PhysicalDescriptor[0] = 0) then
        PhysicalDescriptorLbl.Caption :=  'succeeded (empty descriptor)'
      else
        PhysicalDescriptorLbl.Caption := Format('succeeded (Length=%d)', [I]);
      PhysicalDescriptorBtn.Tag := 2;
    end;
    SetLength(PhysicalDescriptor, I);
  end
  else
    PhysicalDescriptorLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.HIDLanguagesBtnClick(Sender: TObject);
var
  I:   Integer;
  Len: Integer;
  IDs: array [0..253] of WORD;
begin
  HIDLanguagesBtn.Tag := 1;
  if TestDev.OpenFile then
  begin
    // calculate length of StringDescriptor 0
    FillChar(IDs, SizeOf(IDs), $FF);
    Len := 0;
    if HidD_GetIndexedString(TestDev.HidFileHandle, 0, PWideChar(@IDs), SizeOf(IDs)) then
    begin
      for I := High(IDs) downto 0 do
        if IDs[I] <> $FFFF then
        begin
          if IDs[I] = 0 then
            Len := I
          else
            Len := I+1;
          Break;
        end;
      HIDLanguagesLbl.Caption := Format('HidD_GetIndexedString(0) succeeded (%d Languages)', [Len]);
      HIDLanguagesBtn.Tag := 2;
    end
    else
      HIDLanguagesLbl.Caption := Format('HidD_GetIndexedString(0) failed (%s)', [SysErrorMessage(GetLastError)]);
  end
  else
    HIDLanguagesLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.HIDStringsBtnClick(Sender: TObject);
var
  I: Integer;
  N: Integer;
  Buffer: array [0..256] of WideChar;
begin
  HIDStringsBtn.Tag := 1;
  N := 0;
  if TestDev.OpenFile then
  begin
    for I := 1 to 255 do
      if HidD_GetIndexedString(TestDev.HidFileHandle, I, Buffer, SizeOf(Buffer)) then
        Inc(N);
    HIDStringsBtn.Tag := 2;
    HIDStringsLbl.Caption :=  Format('%d strings found', [N]);
  end
  else
    HIDStringsLbl.Caption := 'Could not open device file';
end;

procedure TBasicTestForm.FormDestroy(Sender: TObject);
begin
  if PrepData <> nil then
    HidD_FreePreparsedData(PrepData);
end;

end.
